package net.knarcraft.bookswithoutborders.utility;

import net.knarcraft.bookswithoutborders.BookHoldingState;
import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

public class InventoryHelper {

    /**
     * Gets the book the holder is playing
     * @param player <p>The player holding the book</p>
     * @return <p>The book the player is holding</p>
     */
    public static ItemStack getHeldBook(Player player) {
        BookHoldingState holdingState = getHoldingBookState(player);
        ItemStack heldBook;
        if (holdingState == BookHoldingState.MAIN_HAND) {
            heldBook = getHeldBook(player, true);
        } else if (holdingState == BookHoldingState.OFF_HAND) {
            heldBook = getHeldBook(player, false);
        } else {
            throw new IllegalArgumentException("The player is not holding exactly one book.");
        }
        return heldBook;
    }

    /**
     * Replaces the book the player is holding
     * @param player <p>The player holding the book</p>
     * @param newBook <p>The new book the player should hold</p>
     */
    public static void setHeldBook(Player player, ItemStack newBook) {
        BookHoldingState holdingState = getHoldingBookState(player);
        replaceHeldBook(player, newBook, holdingState == BookHoldingState.MAIN_HAND);
    }

    /**
     * Performs checks to validate that a player contains exactly one book
     * @param player <p>The player to validate</p>
     * @param noBookMessage <p>The message to display if the player is not holding a book</p>
     * @param twoBooksMessage <p>The message to display if the player is holding one book in each hand</p>
     * @return <p>False if the player is holding exactly one book</p>
     */
    public static boolean notHoldingOneBookCheck(Player player, String noBookMessage, String twoBooksMessage) {
        BookHoldingState holdingState = getHoldingBookState(player);

        if (holdingState == BookHoldingState.NONE) {
            BooksWithoutBorders.sendErrorMessage(player, noBookMessage);
            return true;
        }

        if (holdingState == BookHoldingState.BOTH_HANDS) {
            BooksWithoutBorders.sendErrorMessage(player, twoBooksMessage);
            return true;
        }

        return false;
    }

    /**
     * Gets the state of which hands of the player contains a book
     * @param player <p>The player possibly holding a book</p>
     * @return <p>The state of the player's book holding</p>
     */
    public static BookHoldingState getHoldingBookState(Player player) {
        boolean hasBookInMainHand = player.getInventory().getItemInMainHand().getType() == Material.WRITTEN_BOOK;
        boolean hasBookInOffHand = player.getInventory().getItemInOffHand().getType() == Material.WRITTEN_BOOK;

        if (hasBookInMainHand && hasBookInOffHand) {
            return BookHoldingState.BOTH_HANDS;
        } else if (hasBookInMainHand) {
            return BookHoldingState.MAIN_HAND;
        } else if (hasBookInOffHand) {
            return BookHoldingState.OFF_HAND;
        } else {
            return BookHoldingState.NONE;
        }
    }

    /**
     * Gets metadata about the player's held book
     *
     * @param player   <p>The player holding the book</p>
     * @param mainHand <p>Whether to get information about a book in the player's main hand or off hand</p>
     * @return <p>Information about the held book</p>
     */
    public static BookMeta getHeldBookMetadata(Player player, boolean mainHand) {
        return (BookMeta) getHeldBook(player, mainHand).getItemMeta();
    }

    /**
     * Gets the item the player is holding
     * @param player <p>The player to get from</p>
     * @param mainHand <p>Whether to get the item in the player's main hand or off hand</p>
     * @return <p>The item the player is holding in the given hand</p>
     */
    public static ItemStack getHeldBook(Player player, boolean mainHand) {
        if (mainHand) {
            return player.getInventory().getItemInMainHand();
        } else {
            return player.getInventory().getItemInOffHand();
        }
    }

    /**
     * Replaces the player's held item
     *
     * @param player   <p>The player to replace the item for</p>
     * @param newBook  <p>The new book the player should hold</p>
     * @param mainHand <p>Whether to replace the item in the player's main hand or off hand</p>
     */
    public static void replaceHeldBook(Player player, ItemStack newBook, boolean mainHand) {
        if (mainHand) {
            player.getInventory().setItemInMainHand(newBook);
        } else {
            player.getInventory().setItemInOffHand(newBook);
        }
    }

}
